# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division


class ImproperlyConfigured(BaseException):
    """A required configuration option is missing or mis-configured.
    """
